@@ -24,6 +24,10 @@
 #' @param latentNames n.latent length vector of latent variable names 
 #' (used for naming parameters, defaults to eta1, eta2, etc).
 #' 
+#' @param id character string denoting column name containing subject identification variables. 
+#' id data may be of any form, though will be coerced internally to an integer sequence rising from 1.
+#' @param time character string denoting column name containing timing data. Timing data must be numeric.
+#' 
 #' @param T0VAR lower triangular n.latent*n.latent cholesky matrix of latent process initial variance / covariance. 
 #' "auto" freely estimates all parameters.
 #' 
@@ -56,6 +60,9 @@
 #' @param TIpredNames n.TIpred length vector of time independent predictor variable names,
 #' as they appear in the data structure.  Default names are TI1, TI2, etc.
 #' 
+#' @param tipredDefault Logical. TRUE sets any parameters with unspecified time independent 
+#' predictor effects to have effects estimated, FALSE fixes the effect to zero unless individually specified.
+#' 
 #' @param n.TDpred Number of time dependent predictor variables in the dataset.  
 #' 
 #' @param TDpredNames n.TDpred length vector of time dependent predictor variable names, 
@@ -135,7 +142,6 @@
 #' covariances are generated by ctFit.  
 #' Better starting values may improve model fit speed and the chance of an appropriate model fit. 
 #' 
-#' @param timeVarying For type='omx' only. character vector of matrices to allow to vary over measurement occasions. Currently only accepts 'MANIFESTMEANS'. 
 #' @examples 
 #'  ### Frequentist example:
 #'  ### impulse and level change time dependent predictor 
@@ -170,11 +176,12 @@
 #' 
 #' @export
 
-ctModel<-function(LAMBDA, n.manifest = 'auto', n.latent='auto', type='omx', Tpoints=NULL, 
-  manifestNames='auto', latentNames='auto', 
+ctModel<-function(LAMBDA, type='omx',n.manifest = 'auto', n.latent='auto', Tpoints=NULL, 
+  manifestNames='auto', latentNames='auto', id='id',time='time',
   T0VAR="auto", T0MEANS="auto", MANIFESTMEANS="auto", MANIFESTVAR="auto", 
   DRIFT="auto", CINT="auto", DIFFUSION="auto",
-  n.TDpred='auto', TDpredNames='auto', n.TIpred='auto', TIpredNames='auto',
+  n.TDpred='auto', TDpredNames='auto', 
+  n.TIpred='auto', TIpredNames='auto', tipredDefault=TRUE,
   TRAITVAR=NULL, T0TRAITEFFECT=NULL,
   MANIFESTTRAITVAR=NULL,
   TDPREDMEANS="auto", TDPREDEFFECT="auto", 
@@ -183,7 +190,7 @@ ctModel<-function(LAMBDA, n.manifest = 'auto', n.latent='auto', type='omx', Tpoi
   TIPREDMEANS="auto", TIPREDEFFECT="auto", 
   T0TIPREDEFFECT="auto", TIPREDVAR="auto", 
   PARS=NULL,
-  startValues=NULL, timeVarying=NULL){
+  startValues=NULL){
   
   #get dimensions
   if(is.null(n.manifest) || is.null(n.latent) || all(n.manifest %in% 'auto') || all(n.latent %in% 'auto')){
@@ -271,14 +278,15 @@ ctModel<-function(LAMBDA, n.manifest = 'auto', n.latent='auto', type='omx', Tpoi
 
   mats <- ctStanMatricesList()
   for(m in names(mats$base)){
-    if(!is.null(get(m))){
+    if(!is.null(get(m))){ #if the matrix is specified
       val <- get(m)
-      if(!all(val %in% 'auto') && !is.matrix(val)){
-        
-        mat<-ctLabel(TDpredNames=TDpredNames,TIpredNames=TIpredNames,
+      if(!all(val %in% 'auto') && !is.matrix(val)){ #and not auto or a matrix
+        if(m %in% 'PARS') mat <- matrix(PARS,ncol=1) else {
+        mat<-ctLabel(TDpredNames=TDpredNames,TIpredNames=TIpredNames, #set the basic structure
           manifestNames=manifestNames,latentNames=latentNames,matrixname=m,n.latent=n.latent,
           n.manifest=n.manifest,n.TDpred=n.TDpred,n.TIpred=n.TIpred,Tpoints=Tpoints)
-        counter <- 0
+        }
+        counter <- 0 #then start filling with vector
         if(length(val) != length(mat) && length(val) !=1) stop(paste0(m,' needs ',length(mat),' values for ',nrow(mat),' * ',ncol(mat),' matrix, but has ',length(val)))
         if(length(val)==1 && length(mat) > 1) {
           message(m,' specified via single value -- filling ',nrow(mat),' * ',ncol(mat),' matrix:')
@@ -290,7 +298,7 @@ ctModel<-function(LAMBDA, n.manifest = 'auto', n.latent='auto', type='omx', Tpoi
             mat[ri,ci] <- val[counter]
           }
         }
-       
+        
         print(mat,right=TRUE)
         assign(m, mat)
       }
@@ -543,11 +551,11 @@ ctModel<-function(LAMBDA, n.manifest = 'auto', n.latent='auto', type='omx', Tpoi
   ###model checks
   
   #lower triangular check
-  for(tempmatname in c('T0VAR','MANIFESTVAR','TRAITVAR','MANIFESTTRAITVAR','TDPREDVAR','TIPREDVAR')){
+  for(tempmatname in c('T0VAR','MANIFESTVAR','TRAITVAR','MANIFESTTRAITVAR','TDPREDVAR','DIFFUSION','TIPREDVAR')){
     assign('tempmat',get(tempmatname))
     if(is.null(tempmat)) next
     if(all(suppressWarnings(as.numeric(tempmat[upper.tri(tempmat)])) %in% 0)) next
-    warning(paste0(tempmatname,' is not lower triangular! Covariance type matrices should be specified in the appropriate lower-triangular form.'))
+    warning(paste0(tempmatname,' is not lower triangular! Covariance type matrices should usually be specified in the appropriate lower-triangular form.'))
   }
   
  
@@ -568,7 +576,7 @@ ctModel<-function(LAMBDA, n.manifest = 'auto', n.latent='auto', type='omx', Tpoi
     TDPREDEFFECT, TDPREDMEANS, T0TDPREDCOV, TRAITTDPREDCOV,
     TIPREDEFFECT, TIPREDMEANS, T0TIPREDEFFECT,
     TIPREDVAR, TDPREDVAR,TDTIPREDCOV,  PARS, 
-    startValues, timeVarying)
+    startValues,id,time)
   
   names(completemodel)<-c("n.manifest","n.latent","n.TDpred","n.TIpred","Tpoints","LAMBDA",
     "manifestNames","latentNames","TDpredNames","TIpredNames",
@@ -577,14 +585,14 @@ ctModel<-function(LAMBDA, n.manifest = 'auto', n.latent='auto', type='omx', Tpoi
     'TDPREDEFFECT', 'TDPREDMEANS', 'T0TDPREDCOV', 'TRAITTDPREDCOV',
     'TIPREDEFFECT', 'TIPREDMEANS', 'T0TIPREDEFFECT',
     'TIPREDVAR', 'TDPREDVAR','TDTIPREDCOV',  'PARS',
-    "startValues", 'timeVarying')
+    "startValues","id","time")
   
 
   
   
   if(type=='omx') class(completemodel)<-"ctsemInit"
   
-  if(type=='stanct' | type=='standt') completemodel<-ctStanModel(completemodel,type=type)
+  if(type=='stanct' | type=='standt') completemodel<-ctStanModel(completemodel,type=type,tipredDefault= tipredDefault)
   
   return(completemodel)
 }
